package org.cakelab.eval.mlatency;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import org.cakelab.eval.model.DataLog;
import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.MeasurementRun;
import org.cakelab.utils.FileSystem;


/**
 * Our data log stores all data generated by all runs 
 * for the same or similar launch configurations.
 * For each new launch configuration we create a new sub directory &lt;launchconfig&gt;.
 * The name of &lt;launchconfig&gt; consists of the arguments given on the command line.
 * For each run we create a new sub directory &lt;run&gt; in &lt;launchconfig&gt;.
 * The name of &lt;run&gt; consists of the number of the run.
 * &lt;launchconfig&gt;/&lt;run&gt; contains:
 * <ul>
 * <li>stdout.dat: The raw output from standard out</li>
 * <li>stderr.dat: The raw output from standard err</li>
 * <li>.. more to come</li>
 * </ul>
 * @author homac
 *
 */
public class MLatencyDataLog implements DataLog {

	private static final String STDOUT_FILE = "stdout.dat";
	private static final String STDERR_FILE = "stderr.dat";
	private static final String RUN_FILE = "run.dat";
	private String basedir;
	private String datalogDir;
	private String launchConfigDir;
	private String runDir;
	private LaunchConfig launch;
	
	/**
	 * 
	 * @param targetDirectory The base directory for all generated outputs.
	 */
	public MLatencyDataLog(String targetDirectory) {
		this.basedir = targetDirectory;
	}

	@Override
	public void open(LaunchConfig launch) throws IOException {
		this.launch = launch;
		datalogDir = FileSystem.createDataLogDirectory(basedir);
		launchConfigDir = FileSystem.createLaunchConfigDirectory(datalogDir, launch);
	}

	@Override
	public void store(MeasurementRun _run) throws IOException {
		MLatencyMeasurementRun run = (MLatencyMeasurementRun) _run;
		runDir = FileSystem.createRunDir(launchConfigDir, run);

		safeStdout(run);
		
		safeStderr(run);
		
	}
	
	public MeasurementRun load (File frun) throws IOException {
		runDir = frun.getPath();
		MeasurementRun run = MLatencyMeasurementRun.load(launch, frun, createStdOutFilename(), createStdErrFilename());
		return run;
	}
	

	private void safeStderr(MLatencyMeasurementRun run) throws IOException {
		String stderrFilename = createStdErrFilename(); 
		InputStream stderr = run.getStderrStream();
		FileSystem.dump2File(stderr, stderrFilename);
		stderr.close();
		/* provide the run with the new data locations in data store */
		run.setStderrFilename(stderrFilename);
	}

	private String createStdErrFilename() {
		return runDir + File.separator + STDERR_FILE;
	}

	private void safeStdout(MLatencyMeasurementRun run) throws IOException {
		String stdoutFilename = createStdOutFilename(); 
		InputStream stdout = run.getStdoutStream();
		FileSystem.dump2File(stdout, stdoutFilename);
		stdout.close();
		/* provide the run with the new data locations in data store */
		run.setStdoutFilename(stdoutFilename);
	}

	private String createStdOutFilename() {
		return runDir + File.separator + STDOUT_FILE;
	}

	@Override
	public void close() throws IOException {
		/* nothing to do */
	}

	
	class MLatencyDataLogIterator implements Iterator<MeasurementRun>{

		private MLatencyDataLog log;
		private File[] files;
		private int i;

		MLatencyDataLogIterator(MLatencyDataLog log) {
			this.log = log;
			File dir = new File(log.launchConfigDir);
			FilenameFilter filter = new FilenameFilter() {
				@Override
				public boolean accept(File dir, String name) {
					return !name.startsWith(".");
				}};
			files = dir.listFiles(filter);
			i = 0;
		}
		
		@Override
		public boolean hasNext() {
			return i < files.length;
		}

		@Override
		public MeasurementRun next() {
			try {
				return log.load(files[i++]);
			} catch (IOException e) {
				throw new Error(e);
			}
		}

		@Override
		public void remove() {
			if (i < files.length) files[i++].delete();
		}
		
	}
	
	@Override
	public Iterator<MeasurementRun> iterator() {
		return new MLatencyDataLogIterator(this);
	}

}
