package org.cakelab.eval.run;

import java.io.IOException;
import java.util.Vector;

import org.cakelab.eval.mlatency.MLatencyLaunchConfig;
import org.cakelab.eval.model.DataLog;
import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.Measure;
import org.cakelab.eval.model.MeasurementRun;


public class ExecutionControl {
	private LaunchConfig launch;
	private Vector<Measure> measures;
	private Vector<MeasurementRun> runs = new Vector<MeasurementRun>(128);
	private DataLog dataLog;
	
	public ExecutionControl(LaunchConfig launch, Vector<Measure> measures, DataLog dataLog) {
		this.launch = launch;
		this.measures = measures;
		this.dataLog = dataLog;
	}
	
	public ExecutionControl(MLatencyLaunchConfig launch,
			Measure measurement) {
		this.launch = launch;
		this.measures = new Vector<Measure>(1);
		this.measures.add(measurement);
	}

	public void run() throws IOException, InterruptedException {
		Report.info(" *** Evaluating '" + launch.getCommandLine());

		long timestamp = System.currentTimeMillis();
		//
		// Setup measures to initiate the set of runs
		// 
		int i = 0;
		runs.clear();
		for (Measure measure : measures) {
			measure.setup();
		}

		dataLog.open(launch);

		//
		// Read existing data for this run and evaluate confidence
		//
		for (MeasurementRun run : dataLog) {
			runs.add(run);
			updateMeasures(run);
			i++;
		}
		
		while (!isConfident()) {
			Report.info("run: " + i);
			/* let the system calm down to increase accuracy of our next measurement run */
			System.gc();
			Thread.sleep(10);
			
			//
			// execute
			//
			MeasurementRun run = launch.execute();
			//
			// Store raw data first
			//
			dataLog.store(run);
			
			runs.add(run);
			
			//
			// evaluate current confidence intervals
			//
			updateMeasures(run);
			
			i++;
		}
		
		dataLog.close();

		
		//
		// write results
		//
		for (Measure measure : measures) {
			measure.store(timestamp, launch);
		}

		
		//
		// Reset all data structures to initiate the next run
		// 
		for (Measure measure : measures) {
			measure.tearDown();
		}
		
		reportResults();
		
	}

	private void reportResults() {
		Report.info("Execution of '" + launch.getCommandLine() + "' after " + runs.size() + " runs results in:");
		for (Measure measure : measures) {
			Report.info(measure.getName() + " [" +  measure.getUnits() + "]: " + measure.getValues()[1]);
		}
	}

	private void updateMeasures(MeasurementRun run) throws IOException {
		for (Measure measure : measures) {
			measure.add(run);
		}
	}
	
	private boolean isConfident() {
		boolean confident = true;
		for (Measure measure : measures) {
			if (confident) confident = measure.isConfident();
		}
		return confident;
	}
	
}
