package org.cakelab.eval.run;

import org.cakelab.eval.model.ConfidenceEstimator;
import org.cakelab.statistics.NormalDistribution;
import org.cakelab.statistics.Statistics;

/**
 * Relative confidence provides a confidence value standardised to the standard deviation.
 * Example: A relative confidence means, that the confidence interval is less than 20% of 
 * of the standard deviation. This is more reliable than standardisation against full sample 
 * range, which will always contain outliers.
 * 
 * 
 * @author homac
 *
 */
public class RelativeConfidence implements ConfidenceEstimator {

	private double confidenceLevel;
	private double maxDeviationRatio;
	private int minSamples;

	public RelativeConfidence(double confidenceLevel, int minSamples, double maxDeviationRatio) {
		this.confidenceLevel = confidenceLevel;
		this.minSamples = minSamples;
		this.maxDeviationRatio = maxDeviationRatio;
	}
	
	@Override
	public boolean isConfident(Statistics stats) {
		if (stats.getNum() < minSamples) return false;
		double confidenceInterval = getError(stats);
		
		if (Double.isNaN(confidenceInterval)) {
			/* There is no value available for the given configuration. 
			 * This might be intended if there are measurement runs, which apply to other measures except this measure.
			 * So, we just say, that we are confident about the current value .. which is true, because there won't be 
			 * any different value for this measure in the future.
			 * 
			 */
			return true;
		}
		
		
		// TODO: confidence interval relative to stddev is independent of the individual set of samples! Is that right?
		double relativeConfidence = confidenceInterval / stats.getStddev();
		Report.info("width of confidence interval is " + relativeConfidence + " times the standard deviation");
		
		return relativeConfidence <= maxDeviationRatio;
	}

	@Override
	public double getError(Statistics stats) {
		if (stats.getNum() < minSamples) return Double.NaN;
		double confidenceInterval = NormalDistribution.confidenceInterval(confidenceLevel, stats);
		return confidenceInterval;
	}

}
