package org.cakelab.json;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;




/** 
 * This codec encodes and decodes data according to the JavaScript Object Notation.
 * @see <a href="http://www.json.org/">www.json.org</a>
 * @author Holger Machens
 *
 */
public class JSONCodec {

	public String encodeObject(Object o) throws JSONCodecException {
		Object json;
		try {
			json = _encodeObject(o);
			return json.toString();
		} catch (JSONCodecException e) {
			throw e;
		} catch (Exception e) {
			throw new JSONCodecException(e);
		}
	}

	private Object _encodeObject(Object o) throws Exception {
		Object json;
		if (o != null) {
			if (o.getClass().isArray()) {
				json = JSONObject.wrap(o);
			} else {
				json = new JSONObject(o);
			}
		} else {
			json = nullRepresentation();
		}
		return json;
	}

	private Object nullRepresentation() throws JSONException {
		return new JSONObject("{}");
	}

	public Object decodeObject(String encoded, Object target) throws JSONCodecException {
		return _decodeObject(encoded, target);
	}
	
	private Object _decodeObject(String encoded, Object target) throws JSONCodecException {
		JSONObject json;
		try {
			json = new JSONObject(encoded);
			Class<?> type = target.getClass();
			for (Field field : ReflectionHelper.getDeclaredFields(type)) {
				if (json.has(field.getName())) {
					setAttribute(target, field, json);
				}
			}
			return target;
		} catch (JSONException e) {
			throw new JSONCodecException(e);
		}
	}
	

	public void setAttribute(Object target, Field field, JSONObject json) throws JSONCodecException {
		Object o = null;
		Method setter = null;
		try {
			o = json.get(field.getName());
			if (o instanceof JSONArray) {
				setter = ReflectionHelper.findSetter(target, field);
				JSONArray jsonArray = (JSONArray) o;
				Object array = Array.newInstance(field.getType().getComponentType(), jsonArray.length());
				setter.invoke(target, array);
				for (int i = 0; i < jsonArray.length(); i++) {
					Array.set(array, i, jsonArray.get(i));
				}
			} else {
				setter = ReflectionHelper.findSetter(target, field);
				setter.invoke(target, o);
			}
		} catch (IllegalArgumentException e) {
			if (o != null && setter != null) {
				if (o instanceof JSONArray) {
					JSONArray jsonArray = (JSONArray) o;
					if (jsonArray.length() > 0) {
						try {
							o = jsonArray.get(0);
							if (o instanceof String && !field.getType().getComponentType().equals(String.class)) {
								return;
							}
						} catch (JSONException e1) {
							throw new JSONCodecException(e);
						}
					}
				} else if (o instanceof String && !field.getType().equals(String.class)) {
					return;
				}
			}
			throw e;
		} catch (Exception e) {
			throw new JSONCodecException(e);
		}
	}

	public Object decodePrimitive(String body, Class<?> type) {
		if (type.equals(String.class)) {
			return body;
		} else if (type.equals(Character.class)) {
			body = body.trim(); 
			return (body.length() > 0) ? body.charAt(0) : null;
		} else if (type.equals(Long.class)) {
			return Long.decode(body);
		} else if (type.equals(Integer.class)) {
			return Integer.decode(body);
		} else if (type.equals(Double.class)) {
			return Double.parseDouble(body);
		} else if (type.equals(Float.class)) {
			return Float.parseFloat(body);
		} else if (type.equals(Short.class)) {
			return Short.parseShort(body);
		} else if (type.equals(Byte.class)) {
			return Byte.parseByte(body);
		} else if (type.equals(Boolean.class)) {
			return Boolean.parseBoolean(body);
		}
		return null;
	}

	public Object decodeArray(String encoded, Class<?> type) throws JSONCodecException {
		JSONArray json;
		Object target = null;
		try {
			json = new JSONArray(encoded);
			target = Array.newInstance(type.getComponentType(), json.length());
			for (int i = 0; i < json.length(); i++) {
				Array.set(target, i, json.get(i));
			}
			return target;
		} catch (JSONException e) {
			throw new JSONCodecException(e);
		}
	}

	public String encodePrimitive(Object object) {
		return object.toString();
	}

	public String encodeArray(Object array) throws JSONCodecException {
		try {
			if (array == null) {
				return nullRepresentation().toString();
			}
			return new JSONArray(array).toString();
		} catch (JSONException e) {
			throw new JSONCodecException(e);
		}
	}

}
