package org.cakelab.statistics;

import java.util.Arrays;

public class NormalDistribution {
	private static final class QuantileMap {
		/** probablity used as key to find the matching n */
		static final double[] p = new double[]{
			0.80,
			0.90,
			0.95,
			0.98,
			0.99,
			0.995, 
			0.998, 
			0.999, 
			0.9999, 
			0.99999, 
			0.999999, 
			0.9999999, 
			0.99999999, 
			0.999999999
		};
		/** n: factor to sigma */
		static final double[] n = new double[]{
			1.281551565545,
			1.644853626951,
			1.959963984540,
			2.326347874041,
			2.575829303549,
			2.807033768344,
			3.090232306168,
			3.290526731492,
			3.890591886413,
			4.417173413469,
			4.891638475699,
			5.326723886384,
			5.730728868236,
			6.109410204869
		};
	}

	/**
	 * The following method returns values of the quantile function of the
	 * normal distribution. This table is limited to the values required to calculate
	 * confidence intervals between 80% and 99.9999999% probability.
	 * 
	 * Supported values for p are: 
			0.80,
			0.90,
			0.95,
			0.98,
			0.99,
			0.995, 
			0.998, 
			0.999, 
			0.9999, 
			0.99999, 
			0.999999, 
			0.9999999, 
			0.99999999, 
			0.999999999
	 * 
	 * The following table gives the multiple n of σ such that X will lie in 
	 * the range μ ± nσ with a specified probability p. These values are useful 
	 * to determine tolerance interval for sample averages and other statistical 
	 * estimators with normal (or asymptotically normal) distributions.
	 * 
	 * @throws IllegalArgumentException if no quantile value for p was found
	 */
	public static double quantileFunctionValues (double p) {
		int i = Arrays.binarySearch(QuantileMap.p, p);
		if (i < 0) {
			String errmsg = "Probability value to quantile function not supported. Choose one of: ";
			for (double x : QuantileMap.p) {
				errmsg += x + ", ";
			}
			throw new IllegalArgumentException(errmsg) ;
		}
		return QuantileMap.n[i];
	}
	
	/** This method calculates the confidence interval for the mean in a set of samples 
	 * expressed by the given statistics s for a given probability p.
	 * 
	 * @param p Probability of the confidence interval
	 * @param s Statistics on the sample set (especially variance and number of samples)
	 * @return 1/2 of the size of the confidence interval. Thus, the confidence interval around the mean is at mean +/- return value.
	 */
	public static double confidenceInterval (double p, Statistics s) {
		double q = quantileFunctionValues(p);
		return q * s.getStddev() / Math.sqrt(s.getNum());
	}
	
}
